package io.getquill.norm.select

import io.getquill.Spec
import io.getquill.ast.Ident
import io.getquill.ast.Property
import io.getquill.testContext.qr1
import io.getquill.testContext.qr2
import io.getquill.testContext.quote
import io.getquill.testContext.unquote

class ReplaceSelectSpec extends Spec {

  "replaces the final map (select) body" - {
    "simple query" in {
      val q = quote {
        qr1.map(t => t)
      }
      val n = quote {
        qr1.map(t => (t, t.s))
      }
      ReplaceSelect(q.ast, List(Ident("t"), Property(Ident("t"), "s"))) mustEqual
        n.ast
    }
    "nested query" in {
      val q = quote {
        qr1.flatMap(u => qr2.map(t => t.s))
      }
      val n = quote {
        qr1.flatMap(u => qr2.map(t => (t, t.s)))
      }
      ReplaceSelect(q.ast, List(Ident("t"), Property(Ident("t"), "s"))) mustEqual
        n.ast
    }
    "aggregated query" in {
      val q = quote {
        qr1.map(t => t.l).min
      }
      ReplaceSelect(q.ast, List(Property(Ident("t"), "l"))) mustEqual
        q.ast
    }
    "doesn't change the select if not necessary" in {
      val q = quote {
        qr1.map(t => t)
      }
      ReplaceSelect(q.ast, List(Ident("t"))) mustEqual q.ast
    }

    "distinct query" in {
      val q = quote {
        qr1.map(t => t.l).distinct
      }
      ReplaceSelect(q.ast, List(Property(Ident("t"), "l"))) mustEqual
        q.ast
    }
  }

  "fails if the query doesn't have a final map (select)" - {
    "simple query" in {
      val q = quote {
        qr1.filter(t => t.s == "s1")
      }
      intercept[IllegalStateException] {
        ReplaceSelect(q.ast, Nil)
      }
      ()
    }
    "nested query" in {
      val q = quote {
        qr1.flatMap(u => qr2)
      }
      intercept[IllegalStateException] {
        ReplaceSelect(q.ast, Nil)
      }
      ()
    }
  }
}
